# (C) Datadog, Inc. 2024-present
# All rights reserved
# Licensed under a 3-clause BSD style license (see LICENSE)

# Some metrics mapping are too long. This turns off the 120 line limit for this file:
# ruff: noqa: E501


METRIC_MAP = {
    # Operator
    "keda_build_info": "build_info",
    "keda_scaler_active": "scaler.active",
    "keda_scaled_object_paused": "scaled_object.paused",
    "keda_scaler_metrics_value": "scaler.metrics_value",
    "keda_scaler_metrics_latency_seconds": "scaler.metrics_latency_seconds",
    "keda_scaler_detail_errors": "scaler.detail_errors",
    "keda_scaled_object_errors": "scaled_object.errors",
    "keda_scaled_job_errors": "scaled_job.errors",
    "keda_resource_registered_total": "resource_registered",
    "keda_trigger_registered_total": "trigger_registered",
    "keda_internal_scale_loop_latency_seconds": "internal_scale.loop_latency_seconds",
    "keda_cloudeventsource_events_emitted": "cloudeventsource.events_emitted",
    "keda_cloudeventsource_events_queued": "cloudeventsource.events_queued",
    "keda_internal_metricsservice_grpc_server_started": "internal_metricsservice.grpc_server_started",
    "keda_internal_metricsservice_grpc_server_handled": "internal_metricsservice.grpc_server_handled",
    "keda_internal_metricsservice_grpc_server_msg_received": "internal_metricsservice.grpc_server_msg_received",
    "keda_internal_metricsservice_grpc_server_msg_sent": "internal_metricsservice.grpc_server_msg_sent",
    "keda_internal_metricsservice_grpc_server_handling_seconds": "internal_metricsservice.grpc_server_handling_seconds",
    # Operator soon to be deprecated in v2.16
    "keda_scaler_metrics_latency": "scaler.metrics_latency",
    "keda_scaler_errors": "scaler.errors",
    "keda_resource_totals": "resource_totals",
    "keda_trigger_totals": "trigger_totals",
    "keda_internal_scale_loop_latency": "internal_scale.loop_latency",
    # Admission Webhook
    "keda_webhook_scaled_object_validation": "webhook_scaled.object_validation",
    "keda_webhook_scaled_object_validation_errors": "webhook_scaled.object_validation_errors",
    # Metrics Server
    "keda_internal_metricsservice_grpc_client_started": "internal_metricsservice.grpc_client_started",
    "keda_internal_metricsservice_grpc_client_handled": "internal_metricsservice.grpc_client_handled",
    "keda_internal_metricsservice_grpc_client_msg_received": "internal_metricsservice.grpc_client_msg_received",
    "keda_internal_metricsservice_grpc_client_msg_sent": "internal_metricsservice.grpc_client_msg_sent",
    "keda_internal_metricsservice_grpc_client_handling_seconds": "internal_metricsservice.grpc_client_handling_seconds",
    # Metric Server
    "aggregator_discovery_aggregation_count": "aggregator_discovery_aggregation",
    "apiserver_audit_event": "apiserver_audit_event",
    "apiserver_audit_requests_rejected": "apiserver_audit_requests_rejected",
    "apiserver_client_certificate_expiration_seconds": "apiserver_client_certificate_expiration_seconds",
    "apiserver_current_inflight_requests": "apiserver_current_inflight_requests",
    "apiserver_delegated_authz_request": "apiserver_delegated_authz_request",
    "apiserver_delegated_authz_request_duration_seconds": "apiserver_delegated_authz_request_duration_seconds",
    "apiserver_envelope_encryption_dek_cache_fill_percent": "apiserver_envelope_encryption_dek_cache_fill_percent",
    "apiserver_flowcontrol_read_vs_write_current_requests": "apiserver_flowcontrol_read_vs_write_current_requests",
    "apiserver_flowcontrol_seat_fair_frac": "apiserver_flowcontrol_seat_fair_frac",
    "apiserver_request": "apiserver_request",
    "apiserver_request_duration_seconds": "apiserver_request_duration_seconds",
    "apiserver_request_filter_duration_seconds": "apiserver_request_filter_duration_seconds",
    "apiserver_request_sli_duration_seconds": "apiserver_request_sli_duration_seconds",
    "apiserver_request_slo_duration_seconds": "apiserver_request_slo_duration_seconds",
    "apiserver_response_sizes": "apiserver_response_sizes",
    "apiserver_storage_data_key_generation_duration_seconds": "apiserver_storage_data_key_generation_duration_seconds",
    "apiserver_storage_data_key_generation_failures": "apiserver_storage_data_key_generation_failures",
    "apiserver_storage_envelope_transformation_cache_misses": "apiserver_storage_envelope_transformation_cache_misses",
    "apiserver_tls_handshake_errors": "apiserver_tls_handshake_errors",
    "apiserver_webhooks_x509_insecure_sha1": "apiserver_webhooks_x509_insecure_sha1",
    "apiserver_webhooks_x509_missing_san": "apiserver_webhooks_x509_missing_san",
    "authenticated_user_requests": "authenticated_user_requests",
    "authentication_attempts": "authentication_attempts",
    "authentication_duration_seconds": "authentication_duration_seconds",
    "authorization_attempts": "authorization_attempts",
    "authorization_duration_seconds": "authorization_duration_seconds",
    "cardinality_enforcement_unexpected_categorizations": "cardinality_enforcement_unexpected_categorizations",
    "disabled_metrics": "disabled_metrics",
    "field_validation_request_duration_seconds": "field_validation_request_duration_seconds",
    "hidden_metrics": "hidden_metrics",
    "registered_metrics": "registered_metrics",
    # Generic Metrics
    'controller_clientset_k8s_request': 'controller.clientset.k8s.request',
    'go_gc_duration_seconds': 'go.gc.duration.seconds',
    'go_goroutines': 'go.goroutines',
    'go_info': "go.info",
    'go_memstats_alloc_bytes': {'name': 'go.memstats.alloc_bytes', 'type': 'native_dynamic'},
    'go_memstats_buck_hash_sys_bytes': 'go.memstats.buck_hash.sys_bytes',
    'go_memstats_frees': 'go.memstats.frees',
    'go_memstats_gc_cpu_fraction': 'go.memstats.gc.cpu_fraction',
    'go_memstats_gc_sys_bytes': 'go.memstats.gc.sys_bytes',
    'go_memstats_heap_alloc_bytes': 'go.memstats.heap.alloc_bytes',
    'go_memstats_heap_idle_bytes': 'go.memstats.heap.idle_bytes',
    'go_memstats_heap_inuse_bytes': 'go.memstats.heap.inuse_bytes',
    'go_memstats_heap_objects': 'go.memstats.heap.objects',
    'go_memstats_heap_released_bytes': 'go.memstats.heap.released_bytes',
    'go_memstats_heap_sys_bytes': 'go.memstats.heap.sys_bytes',
    'go_memstats_lookups': 'go.memstats.lookups',
    'go_memstats_mallocs': 'go.memstats.mallocs',
    'go_memstats_mcache_inuse_bytes': 'go.memstats.mcache.inuse_bytes',
    'go_memstats_mcache_sys_bytes': 'go.memstats.mcache.sys_bytes',
    'go_memstats_mspan_inuse_bytes': 'go.memstats.mspan.inuse_bytes',
    'go_memstats_mspan_sys_bytes': 'go.memstats.mspan.sys_bytes',
    'go_memstats_next_gc_bytes': 'go.memstats.next.gc_bytes',
    'go_memstats_other_sys_bytes': 'go.memstats.other.sys_bytes',
    'go_memstats_stack_inuse_bytes': 'go.memstats.stack.inuse_bytes',
    'go_memstats_stack_sys_bytes': 'go.memstats.stack.sys_bytes',
    'go_memstats_sys_bytes': 'go.memstats.sys_bytes',
    'go_threads': 'go.threads',
    'process_cpu_seconds': 'process.cpu.seconds',
    'process_max_fds': 'process.max_fds',
    'process_open_fds': 'process.open_fds',
    'process_resident_memory_bytes': 'process.resident_memory.bytes',
    'process_start_time_seconds': {
        'name': 'process.uptime.seconds',
        'type': 'time_elapsed',
    },
    'process_virtual_memory_bytes': 'process.virtual_memory.bytes',
    'process_virtual_memory_max_bytes': 'process.virtual_memory.max_bytes',
    'workqueue_adds': 'workqueue.adds',
    'workqueue_depth': 'workqueue.depth',
    'workqueue_longest_running_processor_seconds': 'workqueue.longest.running_processor.seconds',
    'workqueue_queue_duration_seconds': 'workqueue.queue.duration.seconds',
    'workqueue_retries': 'workqueue.retries',
    'workqueue_unfinished_work_seconds': 'workqueue.unfinished_work.seconds',
    'workqueue_work_duration_seconds': 'workqueue.work.duration.seconds',
    'certwatcher_read_certificate': 'certwatcher.read_certificate',
    'certwatcher_read_certificate_errors': 'certwatcher.read_certificate_errors',
    'controller_runtime_active_workers': 'controller.runtime.active_workers',
    'controller_runtime_max_concurrent_reconciles': 'controller.runtime.max_concurrent_reconciles',
    'controller_runtime_reconcile': 'controller.runtime.reconcile',
    'controller_runtime_reconcile_errors': 'controller.runtime.reconcile_errors',
    'controller_runtime_reconcile_panics': 'controller.runtime.reconcile_panics',
    'controller_runtime_reconcile_time_seconds': 'controller.runtime.reconcile_time.seconds',
    'controller_runtime_terminal_reconcile_errors': 'controller.runtime.terminal_reconcile_errors',
    'controller_runtime_webhook_panics': 'controller.runtime.webhook_panics',
    'controller_runtime_webhook_requests': 'controller.runtime.webhook_requests',
    'controller_runtime_webhook_requests_in_flight': 'controller.runtime.webhook_requests_in_flight',
    'go_memstats_last_gc_time_seconds': {
        'name': 'go.memstats.time_since_last_gc.seconds',
        'type': 'time_elapsed',
    },
    'leader_election_master_status': 'leader_election.master_status',
    'rest_client_requests': 'rest.client.requests',
}

RENAME_LABELS_MAP = {
    'version': 'keda_version',
    'host': 'keda_host',
}
