# (C) Datadog, Inc. 2021-present
# All rights reserved
# Licensed under a 3-clause BSD style license (see LICENSE)

# This file is autogenerated.
# To change this file you should edit assets/configuration/spec.yaml and then run the following commands:
#     ddev -x validate config -s <INTEGRATION_NAME>
#     ddev -x validate models -s <INTEGRATION_NAME>

from __future__ import annotations

from types import MappingProxyType
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, field_validator, model_validator

from datadog_checks.base.utils.functions import identity
from datadog_checks.base.utils.models import validation

from . import defaults, validators


class AgentJobs(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[float] = None
    enabled: Optional[bool] = None
    history_row_limit: Optional[int] = None


class Aws(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    instance_endpoint: Optional[str] = None


class Azure(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    aggregate_sql_databases: Optional[bool] = None
    deployment_type: Optional[str] = None
    fully_qualified_domain_name: Optional[str] = None


class CollectRawQueryStatement(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = None


class CollectSettings(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[float] = None
    enabled: Optional[bool] = None


class CustomQuery(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[int] = None
    columns: Optional[tuple[MappingProxyType[str, Any], ...]] = None
    metric_prefix: Optional[str] = None
    query: Optional[str] = None
    tags: Optional[tuple[str, ...]] = None


class AoMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    ao_database: Optional[str] = None
    availability_group: Optional[str] = None
    enabled: Optional[bool] = Field(None, examples=[False])
    only_emit_local: Optional[bool] = Field(None, examples=[False])


class DbBackupMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[int] = Field(None, examples=[300])
    enabled: Optional[bool] = Field(None, examples=[True])


class DbFilesMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[True])


class DbFragmentationMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[int] = Field(None, examples=[300])
    enabled: Optional[bool] = Field(None, examples=[False])
    enabled_tempdb: Optional[bool] = Field(None, examples=[False])


class DbStatsMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[True])


class FciMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[False])


class FileStatsMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[True])


class IndexUsageMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[int] = Field(None, examples=[300])
    enabled: Optional[bool] = Field(None, examples=[True])
    enabled_tempdb: Optional[bool] = Field(None, examples=[False])


class InstanceMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[True])


class MasterFilesMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[False])


class PrimaryLogShippingMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[False])


class SecondaryLogShippingMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[False])


class ServerStateMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[True])


class TaskSchedulerMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[False])


class TempdbFileSpaceUsageMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[True])


class XeMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    enabled: Optional[bool] = Field(None, examples=[False])


class DatabaseMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    ao_metrics: Optional[AoMetrics] = None
    db_backup_metrics: Optional[DbBackupMetrics] = None
    db_files_metrics: Optional[DbFilesMetrics] = None
    db_fragmentation_metrics: Optional[DbFragmentationMetrics] = None
    db_stats_metrics: Optional[DbStatsMetrics] = None
    fci_metrics: Optional[FciMetrics] = None
    file_stats_metrics: Optional[FileStatsMetrics] = None
    index_usage_metrics: Optional[IndexUsageMetrics] = None
    instance_metrics: Optional[InstanceMetrics] = None
    master_files_metrics: Optional[MasterFilesMetrics] = None
    primary_log_shipping_metrics: Optional[PrimaryLogShippingMetrics] = None
    secondary_log_shipping_metrics: Optional[SecondaryLogShippingMetrics] = None
    server_state_metrics: Optional[ServerStateMetrics] = None
    task_scheduler_metrics: Optional[TaskSchedulerMetrics] = None
    tempdb_file_space_usage_metrics: Optional[TempdbFileSpaceUsageMetrics] = None
    xe_metrics: Optional[XeMetrics] = None


class DeadlocksCollection(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[float] = None
    enabled: Optional[bool] = None
    max_deadlocks: Optional[float] = None


class Gcp(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    instance_id: Optional[str] = None
    project_id: Optional[str] = None


class ManagedIdentity(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    client_id: Optional[str] = None
    identity_scope: Optional[str] = None


class MetricPatterns(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    exclude: Optional[tuple[str, ...]] = None
    include: Optional[tuple[str, ...]] = None


class ObfuscatorOptions(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collect_commands: Optional[bool] = None
    collect_comments: Optional[bool] = None
    collect_metadata: Optional[bool] = None
    collect_tables: Optional[bool] = None
    keep_boolean: Optional[bool] = None
    keep_identifier_quotation: Optional[bool] = None
    keep_null: Optional[bool] = None
    keep_positional_parameter: Optional[bool] = None
    keep_sql_alias: Optional[bool] = None
    keep_trailing_semicolon: Optional[bool] = None
    obfuscation_mode: Optional[str] = None
    remove_space_between_parentheses: Optional[bool] = None
    replace_digits: Optional[bool] = None


class ProcedureMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[float] = None
    dm_exec_procedure_stats_row_limit: Optional[int] = None
    enabled: Optional[bool] = None
    max_procedures: Optional[int] = None


class QueryActivity(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[float] = None
    enabled: Optional[bool] = None


class QueryMetrics(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[float] = None
    disable_secondary_tags: Optional[bool] = None
    dm_exec_query_stats_row_limit: Optional[int] = None
    enabled: Optional[bool] = None
    enforce_collection_interval_deadline: Optional[bool] = None
    lookback_window: Optional[int] = None
    max_queries: Optional[int] = None
    samples_per_hour_per_query: Optional[int] = None


class SchemasCollection(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        frozen=True,
    )
    collection_interval: Optional[float] = None
    enabled: Optional[bool] = None
    max_execution_time: Optional[float] = None


class InstanceConfig(BaseModel):
    model_config = ConfigDict(
        validate_default=True,
        arbitrary_types_allowed=True,
        frozen=True,
    )
    adoprovider: Optional[str] = None
    agent_jobs: Optional[AgentJobs] = None
    autodiscovery_db_service_check: Optional[bool] = None
    autodiscovery_exclude: Optional[tuple[str, ...]] = None
    autodiscovery_include: Optional[tuple[str, ...]] = None
    aws: Optional[Aws] = None
    azure: Optional[Azure] = None
    collect_raw_query_statement: Optional[CollectRawQueryStatement] = None
    collect_settings: Optional[CollectSettings] = None
    command_timeout: Optional[int] = None
    connection_string: Optional[str] = None
    connector: Optional[str] = None
    custom_queries: Optional[tuple[CustomQuery, ...]] = None
    database: Optional[str] = None
    database_autodiscovery: Optional[bool] = None
    database_autodiscovery_interval: Optional[int] = None
    database_instance_collection_interval: Optional[float] = None
    database_metrics: Optional[DatabaseMetrics] = None
    db_fragmentation_object_names: Optional[tuple[str, ...]] = None
    dbm: Optional[bool] = None
    deadlocks_collection: Optional[DeadlocksCollection] = None
    disable_generic_tags: Optional[bool] = None
    driver: Optional[str] = None
    dsn: Optional[str] = None
    empty_default_hostname: Optional[bool] = None
    gcp: Optional[Gcp] = None
    host: str
    ignore_missing_database: Optional[bool] = None
    log_unobfuscated_plans: Optional[bool] = None
    log_unobfuscated_queries: Optional[bool] = None
    managed_identity: Optional[ManagedIdentity] = None
    metric_patterns: Optional[MetricPatterns] = None
    min_collection_interval: Optional[float] = None
    obfuscator_options: Optional[ObfuscatorOptions] = None
    only_custom_queries: Optional[bool] = None
    password: Optional[str] = None
    proc_only_if: Optional[str] = None
    proc_only_if_database: Optional[str] = None
    procedure_metrics: Optional[ProcedureMetrics] = None
    propagate_agent_tags: Optional[bool] = None
    query_activity: Optional[QueryActivity] = None
    query_metrics: Optional[QueryMetrics] = None
    reported_hostname: Optional[str] = None
    schemas_collection: Optional[SchemasCollection] = None
    server_version: Optional[str] = None
    service: Optional[str] = None
    stored_procedure: Optional[str] = None
    stored_procedure_characters_limit: Optional[int] = None
    tags: Optional[tuple[str, ...]] = None
    use_global_custom_queries: Optional[str] = None
    username: Optional[str] = None

    @model_validator(mode='before')
    def _initial_validation(cls, values):
        return validation.core.initialize_config(getattr(validators, 'initialize_instance', identity)(values))

    @field_validator('*', mode='before')
    def _validate(cls, value, info):
        field = cls.model_fields[info.field_name]
        field_name = field.alias or info.field_name
        if field_name in info.context['configured_fields']:
            value = getattr(validators, f'instance_{info.field_name}', identity)(value, field=field)
        else:
            value = getattr(defaults, f'instance_{info.field_name}', lambda: value)()

        return validation.utils.make_immutable(value)

    @model_validator(mode='after')
    def _final_validation(cls, model):
        return validation.core.check_model(getattr(validators, 'check_instance', identity)(model))
